package org.ow2.petals.engine.clock;

import java.io.File;
import java.util.logging.Level;

import javax.jbi.management.DeploymentException;

import org.ow2.petals.component.framework.se.AbstractServiceEngine;

public class Clock extends AbstractServiceEngine {

    private static final String serviceUnit = "serviceUnit";

    private String serviceUnitRootPath;

    private ClockService service;

    private boolean suUndeployed = true;

    public ClockService getService() {
        return service;
    }

    public void doInit() {
        getLogger().info("init Clock service");
        service = new ClockService(getLogger());
        serviceUnitRootPath = context.getInstallRoot() + File.separator + "serviceUnit";
    }

    public void doStart() {
        getLogger().info("start Clock service");
        if (suUndeployed) {
            deploySU();
        }
        startSU();
    }

    public void doStop() {
        getLogger().info("stop Clock service");
        stopSU();
    }

    public void doShutdown() {
        getLogger().info("shutdown Clock service");
        undeploySU();
    }

    private void deploySU() {
        try {
            getLogger().info("deploy embedded ServiceUnit");
            getServiceUnitManager().deploy(serviceUnit, serviceUnitRootPath);
            getServiceUnitManager().init(serviceUnit, serviceUnitRootPath);
            suUndeployed = false;
        } catch (DeploymentException e) {
            getLogger().log(Level.SEVERE, "Error while deploying the embedded ServiceUnit", e);
        }
    }

    private void undeploySU() {
        try {
            getLogger().info("undeploy embedded ServiceUnit");
            getServiceUnitManager().undeploy(serviceUnit, serviceUnitRootPath);
            suUndeployed = true;
        } catch (DeploymentException e) {
            getLogger().log(Level.SEVERE, "Error while undeploying the embedded ServiceUnit", e);
        }
    }

    private void startSU() {
        try {
            getLogger().info("start embedded ServiceUnit");
            getServiceUnitManager().start(serviceUnit);
        } catch (DeploymentException e) {
            getLogger().log(Level.SEVERE,
                    "Error while starting the embedded ServiceUnit, undeploy it", e);
            undeploySU();
        }
    }

    private void stopSU() {
        try {
            getLogger().info("stop embedded ServiceUnit");
            getServiceUnitManager().stop(serviceUnit);
        } catch (DeploymentException e) {
            getLogger().log(Level.SEVERE,
                    "Error while stopping the embedded ServiceUnit, undeploy it", e);
            undeploySU();
        }
    }

}
