package org.ow2.petals.engine.clock;

import java.net.URI;

import javax.jbi.JBIException;
import javax.jbi.messaging.Fault;

import org.apache.commons.lang.CharEncoding;
import org.ow2.petals.component.framework.Constants.MEPConstants;
import org.ow2.petals.component.framework.api.exception.PEtALSCDKException;
import org.ow2.petals.component.framework.api.message.Exchange;
import org.ow2.petals.component.framework.listener.AbstractJBIListener;
import org.ow2.petals.component.framework.util.SourceHelper;

public class Listener extends AbstractJBIListener {

    protected void onJBIMessage(Exchange exchange) {
        ClockService service = ((Clock) getComponent()).getService();

        // Get Operation name
        String opName = exchange.getOperation().getLocalPart();

        // Get Message Exchange Pattern
        URI pattern = exchange.getPattern();
        try {

            if ("time".equalsIgnoreCase(opName)) {
                if (pattern.equals(MEPConstants.IN_OUT_PATTERN.value())
                        || pattern.equals(MEPConstants.IN_OPTIONAL_OUT_PATTERN.value())) {
                    String en = service.time();
                    exchange.setOutMessageContent(en);
                } else {
                    throw new Exception("Operation not allowed (" + "time"
                            + ") with the following pattern (" + exchange.getPattern().toString()
                            + ")");
                }
            } else if ("printTime".equalsIgnoreCase(opName)) {
                if (!pattern.equals(MEPConstants.IN_OUT_PATTERN.value())) {
                    service.printTime();
                } else {
                    throw new Exception("Operation not allowed (" + "printTime"
                            + ") with the following pattern (" + exchange.getPattern().toString()
                            + ")");
                }
            } else {
                throw new Exception("Operation not allowed (" + opName
                        + ") with the following pattern (" + exchange.getPattern().toString() + ")");
            }
        } catch (Exception e) {
            getLogger().severe(e.getMessage());
            if (!pattern.equals(MEPConstants.IN_ONLY_PATTERN.value())) {
                try {
                    setIOFaultOnExchange(exchange, e.getMessage());
                } catch (JBIException e1) {
                    exchange.setError(e1);
                }
            }
        }
    }

    private void setIOFaultOnExchange(Exchange exchange, String ioMessage) throws JBIException {
        StringBuffer faultMessage = new StringBuffer();
        faultMessage
                .append("<?xml version=\"1.0\" encoding=\"UTF-8\"?><tns:ioFault xmlns:tns='org.ow2.petals'><tns:message>");
        faultMessage.append(ioMessage);
        faultMessage.append("</tns:message></tns:ioFault>");
        Fault fault = exchange.createFault();
        try {
            fault
                    .setContent(SourceHelper.createSource(faultMessage.toString(),
                            CharEncoding.UTF_8));
            exchange.setFault(fault);
        } catch (PEtALSCDKException e) {
            throw new JBIException(e.getMessage());
        }
    }
}
